@@ -68,7 +68,7 @@ def _loop_exits_early(loop: nodes.For | nodes.While) -> bool:
     )
 
 
-def _has_abstract_methods(node):
+def _has_abstract_methods(node: nodes.ClassDef) -> bool:
     """Determine if the given `node` has abstract methods.
 
     The methods should be made abstract by decorating them
@@ -189,7 +189,6 @@ class BasicErrorChecker(_BasicChecker):
             "continue-in-finally",
             "Emitted when the `continue` keyword is found "
             "inside a finally clause, which is a SyntaxError.",
-            {"maxversion": (3, 8)},
         ),
         "E0117": (
             "nonlocal name %s found without binding",
@@ -206,6 +205,10 @@ class BasicErrorChecker(_BasicChecker):
         ),
     }
 
+    def open(self) -> None:
+        py_version = self.linter.config.py_version
+        self._py38_plus = py_version >= (3, 8)
+
     @utils.only_required_for_messages("function-redefined")
     def visit_classdef(self, node: nodes.ClassDef) -> None:
         self._check_redefinition("class", node)
@@ -401,7 +404,10 @@ class BasicErrorChecker(_BasicChecker):
                 self.add_message("nonlocal-without-binding", args=(name,), node=node)
                 return
 
-            if name not in current_scope.locals:
+            # Search for `name` in the parent scope if:
+            #  `current_scope` is the same scope in which the `nonlocal` name is declared
+            #  or `name` is not in `current_scope.locals`.
+            if current_scope is node.scope() or name not in current_scope.locals:
                 current_scope = current_scope.parent.scope()
                 continue
 
@@ -409,7 +415,9 @@ class BasicErrorChecker(_BasicChecker):
             return
 
         if not isinstance(current_scope, nodes.FunctionDef):
-            self.add_message("nonlocal-without-binding", args=(name,), node=node)
+            self.add_message(
+                "nonlocal-without-binding", args=(name,), node=node, confidence=HIGH
+            )
 
     @utils.only_required_for_messages("nonlocal-without-binding")
     def visit_nonlocal(self, node: nodes.Nonlocal) -> None:
@@ -492,6 +500,7 @@ class BasicErrorChecker(_BasicChecker):
                 isinstance(parent, nodes.TryFinally)
                 and node in parent.finalbody
                 and isinstance(node, nodes.Continue)
+                and not self._py38_plus
             ):
                 self.add_message("continue-in-finally", node=node)
 
