@@ -2,6 +2,8 @@
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
+from __future__ import annotations
+
 import csv
 import operator
 import platform
@@ -10,13 +12,14 @@ from collections import Counter
 from io import StringIO
 from pathlib import Path
 from typing import Counter as CounterType
-from typing import Dict, List, Optional, TextIO, Tuple, Union
+from typing import TextIO, Tuple
 
 import pytest
 from _pytest.config import Config
 
 from pylint import checkers
 from pylint.config.config_initialization import _config_initialization
+from pylint.constants import IS_PYPY
 from pylint.lint import PyLinter
 from pylint.message.message import Message
 from pylint.testutils.constants import _EXPECTED_RE, _OPERATORS, UPDATE_OPTION
@@ -37,7 +40,7 @@ class LintModuleTest:
     maxDiff = None
 
     def __init__(
-        self, test_file: FunctionalTestFile, config: Optional[Config] = None
+        self, test_file: FunctionalTestFile, config: Config | None = None
     ) -> None:
         _test_reporter = FunctionalTestReporter()
         self._linter = PyLinter()
@@ -45,7 +48,7 @@ class LintModuleTest:
         checkers.initialize(self._linter)
 
         # See if test has its own .rc file, if so we use that one
-        rc_file: Union[Path, str] = PYLINTRC
+        rc_file: Path | str = PYLINTRC
         try:
             rc_file = test_file.option_file
             self._linter.disable("suppressed-message")
@@ -54,19 +57,60 @@ class LintModuleTest:
         except NoFileError:
             pass
 
+        self._test_file = test_file
         try:
             args = [test_file.source]
         except NoFileError:
             # If we're still raising NoFileError the actual source file doesn't exist
             args = [""]
+        if config and config.getoption("minimal_messages_config"):
+            with self._open_source_file() as f:
+                messages_to_enable = {msg[1] for msg in self.get_expected_messages(f)}
+                # Always enable fatal errors
+                messages_to_enable.add("astroid-error")
+                messages_to_enable.add("fatal")
+                messages_to_enable.add("syntax-error")
+            args.extend(["--disable=all", f"--enable={','.join(messages_to_enable)}"])
+
+        # Add testoptions
+        self._linter._arg_parser.add_argument(
+            "--min_pyver", type=parse_python_version, default=(2, 5)
+        )
+        self._linter._arg_parser.add_argument(
+            "--max_pyver", type=parse_python_version, default=(4, 0)
+        )
+        self._linter._arg_parser.add_argument(
+            "--min_pyver_end_position", type=parse_python_version, default=(3, 8)
+        )
+        self._linter._arg_parser.add_argument(
+            "--requires", type=lambda s: [i.strip() for i in s.split(",")], default=[]
+        )
+        self._linter._arg_parser.add_argument(
+            "--except_implementations",
+            type=lambda s: [i.strip() for i in s.split(",")],
+            default=[],
+        )
+        self._linter._arg_parser.add_argument(
+            "--exclude_platforms",
+            type=lambda s: [i.strip() for i in s.split(",")],
+            default=[],
+        )
+        self._linter._arg_parser.add_argument(
+            "--exclude_from_minimal_messages_config", default=False
+        )
+
         _config_initialization(
             self._linter, args_list=args, config_file=rc_file, reporter=_test_reporter
         )
-        self._test_file = test_file
-        self._config = config
+
         self._check_end_position = (
-            sys.version_info >= self._test_file.options["min_pyver_end_position"]
+            sys.version_info >= self._linter.config.min_pyver_end_position
         )
+        # TODO: PY3.9: PyPy supports end_lineno from 3.9 and above
+        if self._check_end_position and IS_PYPY:
+            self._check_end_position = sys.version_info >= (3, 9)  # pragma: no cover
+
+        self._config = config
 
     def setUp(self) -> None:
         if self._should_be_skipped_due_to_version():
@@ -74,30 +118,36 @@ class LintModuleTest:
                 f"Test cannot run with Python {sys.version.split(' ', maxsplit=1)[0]}."
             )
         missing = []
-        for requirement in self._test_file.options["requires"]:
+        for requirement in self._linter.config.requires:
             try:
                 __import__(requirement)
             except ImportError:
                 missing.append(requirement)
         if missing:
             pytest.skip(f"Requires {','.join(missing)} to be present.")
-        except_implementations = self._test_file.options["except_implementations"]
+        except_implementations = self._linter.config.except_implementations
         if except_implementations:
             if platform.python_implementation() in except_implementations:
                 msg = "Test cannot run with Python implementation %r"
                 pytest.skip(msg % platform.python_implementation())
-        excluded_platforms = self._test_file.options["exclude_platforms"]
+        excluded_platforms = self._linter.config.exclude_platforms
         if excluded_platforms:
             if sys.platform.lower() in excluded_platforms:
                 pytest.skip(f"Test cannot run on platform {sys.platform!r}")
+        if (
+            self._config
+            and self._config.getoption("minimal_messages_config")
+            and self._linter.config.exclude_from_minimal_messages_config
+        ):
+            pytest.skip("Test excluded from --minimal-messages-config")
 
     def runTest(self) -> None:
         self._runTest()
 
     def _should_be_skipped_due_to_version(self) -> bool:
         return (
-            sys.version_info < self._test_file.options["min_pyver"]
-            or sys.version_info > self._test_file.options["max_pyver"]
+            sys.version_info < self._linter.config.min_pyver
+            or sys.version_info > self._linter.config.max_pyver
         )
 
     def __str__(self) -> str:
@@ -140,7 +190,7 @@ class LintModuleTest:
     def multiset_difference(
         expected_entries: MessageCounter,
         actual_entries: MessageCounter,
-    ) -> Tuple[MessageCounter, Dict[Tuple[int, str], int]]:
+    ) -> tuple[MessageCounter, dict[tuple[int, str], int]]:
         """Takes two multisets and compares them.
 
         A multiset is a dict with the cardinality of the key as the value.
@@ -168,7 +218,7 @@ class LintModuleTest:
             return open(self._test_file.source, encoding="latin1")
         return open(self._test_file.source, encoding="utf8")
 
-    def _get_expected(self) -> Tuple[MessageCounter, List[OutputLine]]:
+    def _get_expected(self) -> tuple[MessageCounter, list[OutputLine]]:
         with self._open_source_file() as f:
             expected_msgs = self.get_expected_messages(f)
         if not expected_msgs:
@@ -180,8 +230,8 @@ class LintModuleTest:
             ]
         return expected_msgs, expected_output_lines
 
-    def _get_actual(self) -> Tuple[MessageCounter, List[OutputLine]]:
-        messages: List[Message] = self._linter.reporter.messages
+    def _get_actual(self) -> tuple[MessageCounter, list[OutputLine]]:
+        messages: list[Message] = self._linter.reporter.messages
         messages.sort(key=lambda m: (m.line, m.symbol, m.msg))
         received_msgs: MessageCounter = Counter()
         received_output_lines = []
@@ -212,7 +262,7 @@ class LintModuleTest:
         self,
         actual_messages: MessageCounter,
         expected_messages: MessageCounter,
-        actual_output: List[OutputLine],
+        actual_output: list[OutputLine],
     ) -> str:
         msg = [f'Wrong results for file "{self._test_file.base}":']
         missing, unexpected = self.multiset_difference(
@@ -232,17 +282,12 @@ class LintModuleTest:
 
     def error_msg_for_unequal_output(
         self,
-        expected_lines: List[OutputLine],
-        received_lines: List[OutputLine],
+        expected_lines: list[OutputLine],
+        received_lines: list[OutputLine],
     ) -> str:
         missing = set(expected_lines) - set(received_lines)
         unexpected = set(received_lines) - set(expected_lines)
-        error_msg = (
-            f"Wrong output for '{self._test_file.base}.txt':\n"
-            "You can update the expected output automatically with: '"
-            f"python tests/test_functional.py {UPDATE_OPTION} -k "
-            f'"test_functional[{self._test_file.base}]"\'\n\n'
-        )
+        error_msg = f"Wrong output for '{self._test_file.base}.txt':"
         sort_by_line_number = operator.attrgetter("lineno")
         if missing:
             error_msg += "\n- Missing lines:\n"
@@ -252,15 +297,28 @@ class LintModuleTest:
             error_msg += "\n- Unexpected lines:\n"
             for line in sorted(unexpected, key=sort_by_line_number):
                 error_msg += f"{line}\n"
+            error_msg += (
+                "\nYou can update the expected output automatically with:\n'"
+                f"python tests/test_functional.py {UPDATE_OPTION} -k "
+                f'"test_functional[{self._test_file.base}]"\'\n\n'
+                "Here's the update text in case you can't:\n"
+            )
+            expected_csv = StringIO()
+            writer = csv.writer(expected_csv, dialect="test")
+            for line in sorted(received_lines, key=sort_by_line_number):
+                writer.writerow(line.to_csv())
+            error_msg += expected_csv.getvalue()
         return error_msg
 
     def _check_output_text(
         self,
         _: MessageCounter,
-        expected_output: List[OutputLine],
-        actual_output: List[OutputLine],
+        expected_output: list[OutputLine],
+        actual_output: list[OutputLine],
     ) -> None:
-        """This is a function because we want to be able to update the text in LintModuleOutputUpdate."""
+        """This is a function because we want to be able to update the text in
+        LintModuleOutputUpdate.
+        """
         assert expected_output == actual_output, self.error_msg_for_unequal_output(
             expected_output, actual_output
         )
